import numpy as np
import sys, os
from operator import add

target        = '1309-2322'
phasecal      = '1311-2329'
#checksource   = '1259-2310'
project_code  = 'CY6213'
subproject    = 'CY6213_C_001_20180307'
#model_path    = '/mirror2/scratch/jmoldon/emerlin/CY6004/phasecal_model_C/'

label         = '_'.join([subproject,target])+'.'
label2        = '_'.join([subproject,phasecal])+'.'
#label3        = '_'.join([subproject,checksource])+'.'

refant= 'Pi'

do_only_image = False
#do_only_image = True


#_____________________________ Directory structure _____________________________

global_path = '/mirror2/scratch/jmoldon/emerlin/' + project_code + '/'
main_path   = global_path + subproject +'/'

split_path = main_path + 'reduc/splits/'
image_path = main_path + 'reduc/images/'
calib_path = main_path + 'reduc/calib/'
outliers   = main_path + 'reduc/outliers/'
mask_path  = global_path + 'maskpath/'

# Create folder structure
def makedir(pathdir):
    try: os.mkdir(pathdir)
    except: pass

makedir(split_path)
makedir(image_path)
makedir(calib_path)
makedir(outliers)


#################################################################################
###################################    CLEAN    #################################
#################################################################################

def make_imagename(img00, label = '',  num_out = 3):
    imagename   = [image_path+label+img00]
    for i in range(num_out):
        imagename.append(outliers + label+img00+'.outlier.'+str(i))
    return imagename

def doclean(datain, imagename, imageSize, cellSize, threshold, multiscale, uvrange, outertaper, field = '', smallscalebias = 0.6,  antenna = '', maskPath = '', robust = 0.5, timerange = '', interactive = False, gain = 0.1, nterms = 1, spw = '',  gridmode = '', wprojplanes=128, phasecenter = [''], restoringbeam = [], usescratch = False, cyclefactor = 3, dopbcor = False):
    clean(vis   = datain,
    imagename   = imagename,
    mask        = maskPath,
    phasecenter = phasecenter,
    selectdata  = T,
    timerange   = timerange,
    antenna     = antenna,
    field       = field,
    spw         = spw,
    mode        = 'mfs',
    imsize      = imageSize,
    cell        = cellSize,
    weighting   = 'briggs',
    robust      = robust,
    usescratch  = usescratch,
    nterms      = nterms,
    niter       = numberIters,
    gain        = gain,
    threshold   = threshold,
    gridmode    = gridmode,
    wprojplanes = wprojplanes,
    multiscale  = multiscale,
    smallscalebias = smallscalebias,
    uvrange     = uvrange,
    uvtaper     = True,
    outertaper  = outertaper,
    cyclefactor = cyclefactor,
    restoringbeam = restoringbeam,
    interactive = interactive)
    if dopbcor:
        if nterms >= 2: ext = '.tt0'
        else: ext = ''
        imgC1 = imagename[0]
        immath(outfile     = imgC1+'.image.pbcor',
        mode        = 'evalexpr',
        imagename   = [imgC1+'.image'+ext, imgC1+'.flux'],
        expr        = 'IM0[IM1>0.002]/IM1')


def dogaincal(datain, caltable, gaintype, calmode, solint, uvrange, antenna = '', refant = '', spw = '', combine = '', smodel = [], minblperant =4, minsnr=3.0):
    gaincal(vis = datain,
    caltable    = caltable,
    gaintype    = gaintype,
    calmode     = calmode,
    antenna     = antenna,
    solint      = solint ,
    refant      = refant,
    uvrange     = uvrange,
    smodel      = smodel,
    spw         = spw,
    combine     = combine,
    minblperant = minblperant,
    minsnr      = minsnr)


#__________________________ General image parameters __________________________

phasecenter  = ['']
#phasecenter = ['J2000 18h20m21.9386390 +07d11m07.1804060']

numberIters  = 100
taylorTerms1 = 1
taylorTerms2 = 2
myStokes     = 'I'
spw          = ''
timerange    = ''
gain         = 0.05   ##########

refant1    = "ea09"

robust1         = 0.5
robust2         = 0.0
robust3         = 2.0

threshold1      = '0.020mJy'
threshold2      = '0.040mJy'
threshold3      = '0.080mJy'
threshold4      = '0.100mJy'
threshold5      = '0.200mJy'
threshold6      = '0.500mJy'

cellSize1       = ['0.005arcsec','0.005arcsec']
cellSize2       = ['0.008arcsec','0.008arcsec']
cellSize3       = ['0.03arcsec','0.03arcsec']
cellSize4       = ['0.05arcsec','0.05arcsec']

imageSize1      =  [[ 500,  500]]*len(phasecenter)
imageSize2      =  [[1200, 1200]]*len(phasecenter)
imageSize3      =  [[2400, 2400]]*len(phasecenter)
imageSize4      =  [[4096, 4096]]*len(phasecenter)

multiscale0     = []
multiscale1     = [0, 10]
multiscale2     = [0, 10, 30]
multiscale3     = [0, 10, 25, 50, 100, 160]

uvrange_long    = '>35klambda'
uvrange_mid     = '<600klambda'
uvrange_short1  = '<35klambda'
uvrange_short2  = '0~1000klambda'
uvrange_none    = ''

outertaper1     = ''
outertaper2     = ['150klambda']
outertaper3     = ['700klambda']

restoringbeam1  = ['0.5','0.5','0.0deg']

cyclefactor0 = 1
cyclefactor1 = 3
cyclefactor2 = 5



####################
#####  SPLITS  #####
####################

if not do_only_image:
    msfile = main_path + subproject +'_avg.mms'
    split(vis   = msfile,
    keepmms     = False,
    datacolumn  = 'CORRECTED',
    field       = target,
    width       = 2,
    timebin     = '8s',
    keepflags   = False,
    outputvis   = split_path+label+'01'+'.ms')
    statwt(vis=split_path+label+'01'+'.ms', datacolumn='data')

    msfile = main_path + subproject+'_avg.ms'
    split(vis   = msfile,
    datacolumn  = 'CORRECTED',
    field       = phasecal,
    width       = 4,
    timebin     = '12s',
    keepflags   = False,
    outputvis   = split_path+label2+'01'+'.ms')
    statwt(vis=split_path+label2+'01'+'.ms', datacolumn='data')



######################
######  TARGET  ######
######################

phasecenter = ['', 'J2000 13h09m47.696 -23d23m02.31']
imageSize1      =  [[ 500,  500]]*len(phasecenter)
imageSize2      =  [[1200, 1200]]*len(phasecenter)
imageSize3      =  [[2400, 2400]]*len(phasecenter)
imageSize4      =  [[4096, 4096]]*len(phasecenter)

numberIters  = 100


#Scan=4    Field=1311-2329[1] Time=2018/03/08/02:12:30.2500 
#Scan=108  Field=1311-2329[1] Time=2018/03/11/02:02:30.2500 
#Scan=212  Field=1311-2329[1] Time=2018/03/12/02:02:30.2500 
#Scan=316  Field=1311-2329[1] Time=2018/03/13/01:52:31.2500 
#Scan=420  Field=1311-2329[1] Time=2018/03/14/01:52:31.2500 
#Scan=524  Field=1311-2329[1] Time=2018/03/15/01:42:31.2500 
#Scan=628  Field=1311-2329[1] Time=2018/03/16/01:42:31.2500 
#Scan=732  Field=1311-2329[1] Time=2018/03/17/00:36:10.5000 
#Scan=836  Field=1311-2329[1] Time=2018/03/18/01:31:31.2500 
#Scan=940  Field=1311-2329[1] Time=2018/03/19/01:26:30.2500 
#Scan=1044 Field=1311-2329[1] Time=2018/03/20/01:21:31.2500 
#Scan=1148 Field=1311-2329[1] Time=2018/03/21/01:16:30.2500 
#Scan=1252 Field=1311-2329[1] Time=2018/03/22/01:11:31.2500 


t01 = '2018/03/07/20:00:00~2018/03/08/08:00:00'
t02 = '2018/03/10/20:00:00~2018/03/11/08:00:00'
t03 = '2018/03/11/20:00:00~2018/03/12/08:00:00'
t04 = '2018/03/12/20:00:00~2018/03/13/08:00:00'
t05 = '2018/03/13/20:00:00~2018/03/14/08:00:00'
t06 = '2018/03/14/20:00:00~2018/03/15/08:00:00'
t07 = '2018/03/15/20:00:00~2018/03/16/08:00:00'
t08 = '2018/03/16/20:00:00~2018/03/17/08:00:00'
t09 = '2018/03/17/20:00:00~2018/03/18/08:00:00'
t10 = '2018/03/18/20:00:00~2018/03/19/08:00:00'
t11 = '2018/03/19/20:00:00~2018/03/20/08:00:00'
t12 = '2018/03/20/20:00:00~2018/03/21/08:00:00'
t13 = '2018/03/21/20:00:00~2018/03/22/08:00:00'

t_close = '2018/03/10/20:00:00~2018/03/14/08:00:00'

times = [8,11,12,13,14,15,16,17,18,19,20,21,22]

import collections


flux = collections.OrderedDict()
flux_host = collections.OrderedDict()


#________________________________ Image cycle 01 ________________________________
i = 1
datain = split_path+label+'{0:02.0f}'.format(i)+'.ms'
imagename   = make_imagename('icln.'+'{0:02.0f}'.format(i), label, len(phasecenter)-1)
caltable    = calib_path + label + 'gcal.'+'{0:02.0f}'.format(i)

imagename   = make_imagename('icln.'+'{0:02.0f}_00'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '',
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')

imagename   = make_imagename('icln.'+'{0:02.0f}_00b'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t_close,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')

imagename   = make_imagename('icln.'+'{0:02.0f}_01'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t01,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')
imagename   = make_imagename('icln.'+'{0:02.0f}_02'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t02,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')
imagename   = make_imagename('icln.'+'{0:02.0f}_03'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t03,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')
imagename   = make_imagename('icln.'+'{0:02.0f}_04'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t04,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')

imagename   = make_imagename('icln.'+'{0:02.0f}_05'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t05,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')
imagename   = make_imagename('icln.'+'{0:02.0f}_06'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t06,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')
imagename   = make_imagename('icln.'+'{0:02.0f}_07'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t07,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')
imagename   = make_imagename('icln.'+'{0:02.0f}_08'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t08,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')
imagename   = make_imagename('icln.'+'{0:02.0f}_09'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t09,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')


imagename   = make_imagename('icln.'+'{0:02.0f}_10'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t10,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')
imagename   = make_imagename('icln.'+'{0:02.0f}_11'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t11,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')
imagename   = make_imagename('icln.'+'{0:02.0f}_12'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t12,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')
imagename   = make_imagename('icln.'+'{0:02.0f}_13'.format(i), label, len(phasecenter)-1)
doclean(datain, imagename, imageSize1, cellSize1, threshold1, multiscale0,
        uvrange = '', outertaper = '', antenna = '', maskPath = '', timerange=t13,
        nterms = taylorTerms1, gain = gain, phasecenter = phasecenter, robust = 0.5,
        usescratch=False, cyclefactor=cyclefactor0, spw='', interactive= False, dopbcor=False)
flux[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[0]+'.image')
flux_host[imagename[0].split('/')[-1].split('_')[-1]] = imstat(imagename[1]+'.image')




import pickle
f_target = np.asfarray(f_target*factor).T
ef_target = np.asfarray(ef_target*factor).T
f_host = np.asfarray(f_host*factor).T
ef_host = np.asfarray(ef_host*factor).T

pickle.dump(flux, open('flux.pkl', 'wb'))
pickle.dump(flux_host, open('flux_host.pkl', 'wb'))

labels = flux.keys()

f_target= []
f_host= []
ef_target = []
ef_host = []

factor = 2.0

for l in labels:
    try:
      if l not in ['06', '00', '00b']:
        print('{4:3s} {0:5.1f} {1:5.1f} {2:5.1f} {3:5.1f} '.format(flux[l]['max'][0]*1e6*factor, flux[l]['rms'][0]*1e6*factor, flux_host[l]['max'][0]*1e6*factor, flux_host[l]['rms'][0]*1e6*factor, l))
        f_target.append(flux[l]['max']*1e6*factor)
        ef_target.append(flux[l]['rms']*1e6*factor)
        f_host.append(flux_host[l]['max']*1e6*factor)
        ef_host.append(flux_host[l]['rms']*1e6*factor)
    except:
        print('problem with run: ', l)

f_target = np.asfarray(f_target).T[0]
ef_target = np.asfarray(ef_target).T[0]
f_host = np.asfarray(f_host).T[0]
ef_host = np.asfarray(ef_host).T[0]

plt.errorbar(np.arange(len(f_host)), f_host, ef_host)



#CASA <617>: np.average(f_host)
#  Out[617]: 274.93161057160859
#
#CASA <618>: np.std(f_host)
#  Out[618]: 49.802149345040682


#HOST GALAXY:
#Statistical:
#270 +/- 50 microJy/beam
#
#Complete dataset:
#220 +/- 20 microJy/beam
#
#TARGET:
#
#Individual average noise 60 microJy/beam
#
#Complete dataset:
#noise 1sigma = 20 microJy/beam (<60microJy at 3 sigma)
#
#Nearby observation (10-14 March)
#noise 1sigma = 34 microJy/beam (<100 microJy at 3 sigma)


plt.clf()
plt.errorbar(np.arange(len(f_host))+1, f_host, ef_host, marker='o', color='k')

plt.xlabel('Run')
plt.xlim(0, 13)
plt.ylabel('Flux density [microJy]')
plt.ylim(0, 500)
plt.title('NGC 4339')



#factor = 2.0
#00   74.3  17.9 216.0  19.8 
#00b 143.8  34.4 177.2  35.0 
#01  248.8  63.0 248.8  61.4 
#02  207.6  56.2 246.7  53.5 
#03  396.1  90.1 339.3  96.1 
#04  372.0  87.0 278.3  85.4 
#05  216.7  54.5 222.5  54.6 
#07  259.6  56.1 392.8  58.7 
#08  242.3  63.4 261.9  68.3 
#09  226.6  58.2 266.2  59.3 
#10  255.1  67.1 263.5  68.7 
#11  249.5  65.9 246.3  64.7 
#12  228.6  55.2 210.9  53.5 
#13  282.9  62.7 321.9  61.9 

